<?php namespace Backend\Behaviors\FormController;

use Backend;
use SystemException;
use ApplicationException;

/**
 * HasFormDesigns
 */
trait HasFormDesigns
{
    /**
     * formGetDesignFormSize is an API call used by ListController
     */
    protected function formGetDesignFormSize()
    {
        return $this->getDesignFormSize();
    }

    /**
     * getDesignDisplayMode
     */
    protected function getDesignDisplayMode()
    {
        return $this->getConfig(
            "{$this->context}[design][displayMode]",
            $this->getConfig('design[displayMode]')
        );
    }

    /**
     * getDesignFormSize
     */
    protected function getDesignFormSize()
    {
        $value = $this->getConfig(
            "{$this->context}[design][size]",
            $this->getConfig('design[size]')
        ) ?: 'auto';

        return Backend::sizeToPixels($value) ?: null;
    }

    /**
     * getDesignBodyClass
     */
    protected function getDesignBodyClass()
    {
        if ($this->getDesignDisplayMode() === 'sidebar') {
            return  'compact-container';
        }

        return null;
    }

    /**
     * isHorizontalForm
     */
    protected function isHorizontalForm(): bool
    {
        if ($this->getConfig("{$this->context}[design][horizontalMode]", $this->getConfig('design[horizontalMode]'))) {
            return true;
        }

        return $this->getDesignDisplayMode() === 'survey';
    }

    /**
     * isSurveyDesign
     */
    protected function isSurveyDesign(): bool
    {
        if ($this->getConfig("{$this->context}[design][surveyMode]", $this->getConfig('design[surveyMode]'))) {
            return true;
        }

        return $this->getDesignDisplayMode() === 'survey';
    }

    /**
     * isPopupDesign
     */
    protected function isPopupDesign(): bool
    {
        return $this->getDesignDisplayMode() === 'popup';
    }

    /**
     * hidePopupDesign
     */
    protected function hidePopupDesign()
    {
        $this->extensionHideMethod('index_onPopupLoadForm');
        $this->extensionHideMethod('index_onPopupSave');
        $this->extensionHideMethod('index_onPopupCancel');
    }

    /**
     * index_onPopupLoadForm
     */
    public function index_onLoadPopupForm()
    {
        if (!$this->isPopupDesign()) {
            throw new ApplicationException(__("This form is not using a popup design."));
        }

        if ($id = post('record_id')) {
            $this->update($id);
            $this->vars['popupTitle'] = $this->getLang('update[title]', 'backend::lang.form.update_title');
            $this->vars['recordId'] = $id;
        }
        else {
            $this->create();
            $this->vars['popupTitle'] = $this->getLang('create[title]', 'backend::lang.form.create_title');
        }

        return $this->formRenderDesign();
    }

    /**
     * index_onSave
     */
    public function index_onPopupSave()
    {
        if ($id = post('record_id')) {
            $this->update_onSave($id);
        }
        else {
            $this->create_onSave();
        }

        return $this->controller->listRefresh();
    }

    /**
     * index_onPopupCancel
     */
    public function index_onPopupCancel()
    {
        if ($id = post('record_id')) {
            $this->update_onCancel($id);
        }
        else {
            $this->create_onCancel();
        }
    }

    /**
     * index_onPopupDelete
     */
    public function index_onPopupDelete()
    {
        if ($id = post('record_id')) {
            $this->update_onDelete($id);
        }

        return $this->controller->listRefresh();
    }
}
