<?php namespace Backend\Behaviors\RelationController;

use October\Rain\Html\Helper as HtmlHelper;

/**
 * HasNestedRelations implements nested relation chains
 */
trait HasNestedRelations
{
    /**
     * @var array manageIds provided by the relationship chain
     */
    protected $manageIds = [];

    /**
     * initNestedRelation checks the extra configuration for a relationship chain
     * and binds the manage forms to the controller, which may contain additional
     * relation definitions via the relation form widget.
     */
    protected function initNestedRelation($model, $parentField)
    {
        $checked = [];
        $checked[$parentField] = true;

        $chain = $this->extraConfig['chain'] ?? [];
        foreach ($chain as $info) {
            @[$field, $manageId] = $info;

            if (!$field || isset($checked[$field])) {
                continue;
            }

            if ($manageId) {
                $this->manageIds[$field] = $manageId;
            }

            $this->initRelationInternal($model, $field);

            $checked[$field] = true;
        }
    }

    /**
     * makeNestedRelationModel resolves a relation based on a nested field name
     * E.g: model[relation1][relation2] → $model->relation1()->relation2()
     */
    protected function makeNestedRelationModel($model, $field)
    {
        if (!str_contains($field, '[') || !str_contains($field, ']')) {
            return [$model, $field];
        }

        $parts = HtmlHelper::nameToArray($field);
        $lastField = array_pop($parts);
        while ($rootField = array_shift($parts)) {
            $model = $model->$rootField()->getRelated();
        }

        if (
            isset($this->manageIds[$field]) &&
            ($manageModel = $model->find($this->manageIds[$field]))
        ) {
            $model = $manageModel;
        }
        elseif (
            $this->manageId &&
            ($manageModel = $model->find($this->manageId))
        ) {
            $model = $manageModel;
        }

        return [$model, $lastField];
    }
}
